# frozen_string_literal: true

require 'apple_core/refinements/hash'

require 'erratum/configuration'
require 'erratum/error'
require 'erratum/errors/authentication'
require 'erratum/errors/authentication/duplicate_authentication'
require 'erratum/errors/authentication/invalid_token'
require 'erratum/errors/authentication/invalid_username_or_password'
require 'erratum/errors/authorization'
require 'erratum/errors/authorization/forbidden'
require 'erratum/errors/crud'
require 'erratum/errors/crud/attribute_length'
require 'erratum/errors/crud/invalid_association'
require 'erratum/errors/crud/resource_not_found'
require 'erratum/errors/crud/resource_not_unique'
require 'erratum/errors/crud/resource_persistence'
require 'erratum/errors/request/parameter_missing'
require 'erratum/errors/request/unpermitted_parameters'
require 'erratum/errors/runtime_error'
require 'erratum/rescuable_resource'
require 'erratum/verifiable_resource'
require 'erratum/version'

module  Erratum
  using ::AppleCore::Refinements::Hash

  # rubocop:disable Metrics/CyclomaticComplexity, Metrics/PerceivedComplexity
  def self.build(original_error, overrides = {})
    overrides           = overrides.symbolize_keys
    original_error_name = error_name(original_error)
    build_error_name    = mapped_error_name(original_error_name)
    build_error_class   = error_class(build_error_name)

    if original_error.is_a?(Class) &&
       original_error < ::Erratum::Error

      original_error.new(overrides)
    elsif original_error.is_a?(String)
      build_error_class.new(overrides)
    elsif original_error.is_a?(::Erratum::Error)
      original_source = original_error.source.symbolize_keys

      build_error_class.new(original_source.merge(overrides))
    elsif build_error_class.respond_to?(:convert)
      build_error_class.convert(original_error, overrides)
    elsif build_error_class < Exception &&
          build_error_class == original_error.class
      ::Erratum::Errors::RuntimeError.wrap(original_error)
    else
      ::Erratum::Errors::RuntimeError.new(
        message: "#{build_error_class.name}: #{original_error.inspect}",
      )
    end
  end
  # rubocop:enable Metrics/CyclomaticComplexity, Metrics/PerceivedComplexity

  def self.raise(error_type, **args)
    Kernel.fail build(error_type, **args)
  end

  def self.fail(error_type, **args)
    Kernel.fail build(error_type, **args)
  end

  alias fail raise

  def self.error_name(error)
    if error.is_a?(String)
      error.gsub(/\A::/, '')
    elsif error.is_a? Class
      error.name
    else
      error.class.name
    end
  end

  def self.mapped_error_name(name)
    configuration
      .error_mappings
      .fetch(name, name)
      .gsub(/\A::/, '')
  end

  def self.error_class(name)
    if const_defined?("::#{name}")
      Object.const_get(name)
    else
      Object.const_get("Erratum::Errors::#{name}")
    end
  rescue NameError => error
    Kernel.raise(
      error.exception(<<~HEREDOC.chomp.tr("\n", ' ')),
        Erratum could not find an error class with either the name "#{name}" or
        with the name "Erratum::Errors::#{name}". Please verify the spelling and
        that the class is loaded.
      HEREDOC
    )
  end
end
