# frozen_string_literal: true

require 'erratum/error'
require 'erratum/errors/crud'

module  Erratum
module  Errors
class   ResourceNotFound < RuntimeError
  include Error
  include Errors::Crud

  attr_accessor :missing_ids

  def self.convert(original_error, overrides = {})
    initialization_parameters = {}

    case original_error.class.name
    when 'ActiveRecord::RecordNotFound'
      initialization_parameters = {
        missing_ids: case original_error.message
                     when /\ACouldn't find .* without an ID\z/
                       []
                     when /\ACouldn't find .* with \'.*\'=([a-f0-9\-]+)/
                       [Regexp.last_match(1)]
                     when /\ACouldn't find all .* with \'.*\': ((?:[a-f0-9\-]+(?:, )?)+)/
                       Array(Regexp.last_match(1).split(', '))
                     end,
      }
    end

    new(initialization_parameters.merge(overrides))
  end

  def http_status
    404
  end

  def title
    'Resource Not Found'
  end

  def detail
    <<~HEREDOC.chomp.tr("\n", ' ')
      The #{resource_name} you attempted to #{action} for this request is either
      not authorized for the authenticated user or does not exist.
    HEREDOC
  end

  def source
    super.merge(
      'missing_ids' => missing_ids,
    )
  end

  def action
    @action || 'access'
  end
end
end
end
