# frozen_string_literal: true

require 'apple_core/refinements/string'
require 'erratum/error'
require 'erratum/errors/crud'

module  Erratum
module  Errors
class   ResourcePersistence < RuntimeError
  using   ::AppleCore::Refinements::String

  include Error
  include Errors::Crud

  attr_accessor :pointer,
                :attribute,
                :value

  def self.convert(original_error, overrides = {})
    case original_error.class.name
    when 'ActiveRecord::RecordInvalid',
         'ActiveRecord::RecordNotSaved'

      original_error.record.errors.each_with_object([]) do |(attribute, error), errors|
        initialization_parameters = if attribute == :base
                                      {
                                        attribute: nil,
                                        detail:    error.to_s,
                                        pointer:   '/data/base',
                                        value:     nil,
                                      }
                                    else
                                      {
                                        attribute: attribute.to_s.camelize(:lower),
                                        detail:    "#{attribute.to_s.humanize} #{error}",
                                        pointer:   "/data/attributes/#{attribute}",
                                        value:     original_error
                                                     .record
                                                     .public_send(attribute)
                                                     .to_s,
                                      }
                                    end

        errors << new(initialization_parameters.merge(overrides))
      end
    end
  end

  def http_status
    422
  end

  def title
    'Resource Persistence Error'
  end

  def detail
    @detail || <<~HEREDOC.chomp.tr("\n", ' ')
      One or more of the attributes on the #{resource_name} you attempted to
      #{action} is invalid.
    HEREDOC
  end

  def source
    super.merge(
      'pointer'   => pointer,
      'parameter' => attribute,
      'value'     => value,
    )
  end
end
end
end
